@@ -3,6 +3,7 @@
 """Implements error-handling according to XCP spec.
 """
 import functools
+import logging
 import os
 import threading
 import time
@@ -16,10 +17,10 @@ from pyxcp.types import XcpError
 from pyxcp.types import XcpResponseError
 from pyxcp.types import XcpTimeoutError
 
-from ..logger import Logger
 
 handle_errors = True  # enable/disable XCP error-handling.
 
+logger = logging.getLogger("pyxcp.errorhandler")
 
 class SingletonBase(object):
     _lock = threading.Lock()
@@ -143,6 +144,7 @@ class Repeater:
 
     def __init__(self, initial_value: int):
         self._counter = initial_value
+        #print("\tREPEATER ctor", hex(id(self)))
 
     def repeat(self):
         """Check if repetition is required.
@@ -151,7 +153,7 @@ class Repeater:
         -------
             bool
         """
-        # print("\t\tCOUNTER:", self._counter)
+        #print("\t\tCOUNTER:", hex(id(self)), self._counter)
         if self._counter == Repeater.INFINITE:
             return True
         elif self._counter > 0:
@@ -172,7 +174,7 @@ def display_error():
 class Handler:
     """"""
 
-    logger = Logger(__name__)
+    logger = logger
 
     def __init__(self, instance, func, arguments, error_code=None):
         self.instance = instance
@@ -184,6 +186,7 @@ class Handler:
         self.arguments = arguments
         self.service = self.instance.service
         self.error_code = error_code
+        self._repeater = None
 
     def __str__(self):
         return "Handler(func = {} arguments = {} service = {} error_code = {})".format(
@@ -191,8 +194,20 @@ class Handler:
         )
 
     def __eq__(self, other):
+        if other is None:
+            return False
         return (self.instance == other.instance) and (self.func == other.func) and (self.arguments == other.arguments)
 
+    @property
+    def repeater(self):
+        #print("\tGet repeater", hex(id(self._repeater)), self._repeater is None)
+        return self._repeater
+
+    @repeater.setter
+    def repeater(self, value):
+        #print("\tSet repeater", hex(id(value)))
+        self._repeater = value
+
     def execute(self):
         self.logger.debug("EXECUTE func = {} arguments = {})".format(func_name(self.func), self.arguments))
         if isinstance(self.func, types.MethodType):
@@ -272,7 +287,8 @@ class HandlerStack:
         self._stack = []
 
     def push(self, handler):
-        self._stack.append(handler)
+        if handler != self.tos():
+            self._stack.append(handler)
 
     def pop(self):
         if len(self) > 0:
@@ -308,7 +324,7 @@ class Executor(SingletonBase):
     handlerStack = HandlerStack()
     repeater = None
     newHandler = True
-    logger = Logger(__name__)
+    logger = logger
     previous_error_code = None
     error_code = None
     func = None
@@ -321,40 +337,52 @@ class Executor(SingletonBase):
         self.arguments = arguments
         handler = Handler(inst, func, arguments)
         self.handlerStack.push(handler)
-        while True:
-            try:
-                handler = self.handlerStack.tos()
-                res = handler.execute()
-            except XcpResponseError as e:
-                self.logger.error("XcpResponseError [{}]".format(str(e)))
-                if self.newHandler:
-                    self.error_code = e.get_error_code()
-            except XcpTimeoutError:
-                self.logger.error("XcpTimeoutError")
-                if self.newHandler:
-                    self.error_code = XcpError.ERR_TIMEOUT
-            except Exception as e:
-                raise UnrecoverableError("Don't know how to handle exception '{}'".format(repr(e))) from e
-            else:
-                self.error_code = None
-                # print("\t\t\t*** SUCCESS ***")
-                self.handlerStack.pop()
-                if self.handlerStack.empty():
-                    # print("OK, all handlers passed: '{}'.".format(res))
-                    return res
-            if self.error_code is not None:
-                preActions, actions, repeater = handler.actions(*getActions(inst.service, self.error_code))
-                self.repeater = repeater
-                for f, a in reversed(preActions):
-                    self.handlerStack.push(Handler(inst, f, a, self.error_code))
-                self.newHandler = False
-                continue
-            self.previous_error_code = self.error_code
-            if self.repeater:
-                if self.repeater.repeat():
-                    continue
+        #print("\tENTER handler:", hex(id(handler)))
+        try:
+            while True:
+                try:
+                    handler = self.handlerStack.tos()
+                    #print("\t\tEXEC", hex(id(handler)))
+                    res = handler.execute()
+                except XcpResponseError as e:
+                    self.logger.error("XcpResponseError [{}]".format(str(e)))
+                    if self.newHandler:
+                        self.error_code = e.get_error_code()
+                except XcpTimeoutError as e:
+                    self.logger.error("XcpTimeoutError [{}]".format(str(e)))
+                    if self.newHandler:
+                        self.error_code = XcpError.ERR_TIMEOUT
+                except Exception as e:
+                    raise UnrecoverableError("Don't know how to handle exception '{}'".format(repr(e))) from e
                 else:
-                    raise UnrecoverableError("Finish for now.")
+                    self.error_code = None
+                    # print("\t\t\t*** SUCCESS ***")
+                    self.handlerStack.pop()
+                    if self.handlerStack.empty():
+                        # print("OK, all handlers passed: '{}'.".format(res))
+                        return res
+
+                if self.error_code is not None and self.newHandler is True:
+                    preActions, actions, repeater = handler.actions(*getActions(inst.service, self.error_code))
+                    if handler.repeater is None:
+                        handler.repeater = repeater
+                    for f, a in reversed(preActions):
+                        self.handlerStack.push(Handler(inst, f, a, self.error_code))
+                self.previous_error_code = self.error_code
+                if handler.repeater:
+                    if handler.repeater.repeat():
+                        continue
+                    else:
+                        raise UnrecoverableError(f"Max. repetition count reached while trying to execute service '{handler.func.__name__}'.")
+        finally:
+            # cleanup of class variables
+            self.newHandler = True
+            self.previous_error_code = None
+            while not self.handlerStack.empty():
+                self.handlerStack.pop()
+            self.error_code = None
+            self.func = None
+            self.arguments = None
 
 
 def disable_error_handling(value: bool):
