@@ -82,6 +82,7 @@ class Master:
         ),  # Bypass error-handling for performance reasons.
         "SEED_N_KEY_DLL": (str, False, ""),
         "SEED_N_KEY_DLL_SAME_BIT_WIDTH": (bool, False, False),
+        "DISCONNECT_RESPONSE_OPTIONAL": (bool, False, False),
     }
 
     def __init__(self, transportName, config=None):
@@ -117,6 +118,7 @@ class Master:
         self.currentProtectionStatus = None
         self.seedNKeyDLL = self.config.get("SEED_N_KEY_DLL")
         self.seedNKeyDLL_same_bit_width = self.config.get("SEED_N_KEY_DLL_SAME_BIT_WIDTH")
+        self.disconnect_response_optional = self.config.get("DISCONNECT_RESPONSE_OPTIONAL")
         self.slaveProperties = SlaveProperties()
         self.slaveProperties.pgmProcessor = SlaveProperties()
 
@@ -235,9 +237,16 @@ class Master:
 
         Note
         -----
-        If DISCONNECT is currently not possible, ERR_CMD_BUSY will be returned.
+        - If DISCONNECT is currently not possible, ERR_CMD_BUSY will be returned.
+        - While XCP spec. requires a response, this behavior can be made optional by adding
+            - `DISCONNECT_RESPONSE_OPTIONAL = true` (TOML)
+            - `"DISCONNECT_RESPONSE_OPTIONAL": true` (JSON)
+            to your configuration file.
         """
-        response = self.transport.request(types.Command.DISCONNECT)
+        if self.disconnect_response_optional:
+            response = self.transport.request_optional_response(types.Command.DISCONNECT)
+        else:
+            response = self.transport.request(types.Command.DISCONNECT)
         # self.connected = False
         return response
 
@@ -499,7 +508,7 @@ class Master:
         ----
         For details refer to XCP specification.
         """
-        return self.transport.request(types.Command.TRANSPORT_LAYER_CMD, subCommand, *data)
+        return self.transport.request_optional_response(types.Command.TRANSPORT_LAYER_CMD, subCommand, *data)
 
     @wrapped
     def userCmd(self, subCommand: int, data: bytes):
@@ -680,13 +689,21 @@ class Master:
             chunk_size = max_payload
             chunks = range(total_length // chunk_size)
             remaining = total_length % chunk_size
+            percent_complete = 1
+            callback_remaining = total_length
             for _ in chunks:
                 block = data[offset : offset + max_payload]
-                self.download(block, max_payload)
+                dl_func(block, max_payload, last=True)
                 offset += max_payload
+                callback_remaining -= chunk_size
+                if callback and callback_remaining <= total_length - (total_length / 100) * percent_complete:
+                    callback(percent_complete)
+                    percent_complete += 1
             if remaining:
                 block = data[offset : offset + remaining]
-                self.download(block, remaining)
+                dl_func(block, remaining, last=True)
+                if callback:
+                    callback(percent_complete)
 
     def _block_downloader(self, data: bytes, dl_func=None, dl_next_func=None, minSt=0):
         """Re-usable block downloader.
@@ -732,7 +749,7 @@ class Master:
             delay(minSt)
 
     @wrapped
-    def download(self, data: bytes, blockModeLength=None, **kwargs):
+    def download(self, data: bytes, blockModeLength=None, last=False):
         """Transfer data from master to slave.
 
         Parameters
@@ -749,7 +766,7 @@ class Master:
         Adress is set via :meth:`setMta`
         """
 
-        if blockModeLength is None:
+        if blockModeLength is None or last:
             # standard mode
             length = len(data)
             response = self.transport.request(types.Command.DOWNLOAD, length, *data)
@@ -1281,9 +1298,12 @@ class Master:
             return None
 
     @wrapped
-    def programReset(self):
+    def programReset(self, wait_for_optional_response=True):
         """Indicate the end of a programming sequence."""
-        return self.transport.request_optional_response(types.Command.PROGRAM_RESET)
+        if wait_for_optional_response:
+            return self.transport.request_optional_response(types.Command.PROGRAM_RESET)
+        else:
+            return self.transport.block_request(types.Command.PROGRAM_RESET)
 
     @wrapped
     def getPgmProcessorInfo(self):
@@ -1765,6 +1785,26 @@ class Master:
             else:
                 raise SeedNKeyError("SeedAndKey DLL returned: {}".format(SeedNKeyResult(result).name))
 
+    def identifier(self, id_value: int) -> str:
+        """Return the identifier for the given value.
+        Use this method instead of calling `getId()` directly.
+
+        Parameters
+        ----------
+        id_value: int
+            For standard identifiers, use the constants from `pyxcp.types.XcpGetIdType`.
+
+        Returns
+        -------
+        str
+        """
+        gid = self.getId(id_value)
+        if (gid.mode & 0x01) == 0x01:
+            value = bytes(gid.identification or b"")
+        else:
+            value = self.fetch(gid.length)
+        return value.decode("utf-8")
+
     def id_scanner(self, scan_ranges: Optional[Collection[Collection[int]]] = None) -> Dict[str, str]:
         """Scan for available standard identification types (GET_ID).
 
@@ -1823,15 +1863,14 @@ class Master:
         for id_value, name in gen:
             response = b""
             try:
-                gid = self.getId(id_value)
-                response = self.fetch(gid.length)
+                response = self.identifier(id_value)
             except types.XcpResponseError:
                 # don't depend on confirming implementation, i.e.: ID not implemented ==> empty response.
                 pass
             except Exception:
                 raise
             if response:
-                result[name] = response.decode("utf8")
+                result[name] = response
         return result
 
 
