@@ -9,8 +9,8 @@
 
 #include <array>
 #include <atomic>
+#include <bitset>
 #include <exception>
-#include <coroutine>
 #include <functional>
 #include <optional>
 #include <condition_variable>
@@ -22,6 +22,7 @@
 
 #include <cerrno>
 #include <cstdint>
+#include <cstdlib>
 #include <cstring>
 #include <cstdio>
 
@@ -29,7 +30,6 @@
 #include <thread>
 #include <vector>
 
-//#include "coro/mcnellis_generator.h"
 
 #if defined(_WIN32)
     #include <io.h>
@@ -38,7 +38,6 @@
     #include <Windows.h>
 #endif /* _WIN32 */
 
-#include <stdlib.h>
 
 #include "lz4.h"
 #include "mio.hpp"
@@ -50,15 +49,6 @@
 
     namespace py = pybind11;
     using namespace pybind11::literals;
-
-    struct DType {
-        uint8_t category {0};
-        uint16_t counter {0};
-        double timestamp  {0.0};
-        uint8_t payload[];
-    };
-
-
 #endif /* STANDALONE_REKORDER */
 
 #if !defined(__BIGGEST_ALIGNMENT__)
@@ -80,6 +70,7 @@ constexpr auto megabytes(std::size_t value) -> std::size_t
     return kilobytes(value) * 1024;
 }
 
+constexpr uint16_t XCP_PAYLOAD_MAX = 0xFFFF;
 
 /*
 byte-order is, where applicable little ending (LSB first).
@@ -97,6 +88,8 @@ struct FileHeaderType
     uint32_t size_uncompressed;
 };
 
+using HeaderTuple = std::tuple<std::uint32_t, std::uint32_t, std::uint32_t, std::uint32_t, double>;
+
 static_assert(sizeof(FileHeaderType) == 22);
 
 struct ContainerHeaderType
@@ -106,11 +99,10 @@ struct ContainerHeaderType
     uint32_t size_uncompressed;
 };
 
-using blob_t = char;    // Signedness doesn't matter in this use-case.
+using blob_t = unsigned char;
 
 #if STANDALONE_REKORDER == 1
     using payload_t = std::shared_ptr<blob_t[]>;
-    //using payload_t = std::unique_ptr<blob_t[]>;
 #else
     using payload_t = py::array_t<blob_t>;
 #endif /* STANDALONE_REKORDER */
@@ -128,6 +120,8 @@ struct frame_header_t
 using FrameTuple = std::tuple<std::uint8_t, std::uint16_t, double, std::uint16_t, payload_t>;
 using FrameVector = std::vector<FrameTuple>;
 
+using FrameTupleWriter = std::tuple<std::uint8_t, std::uint16_t, double, std::uint16_t, char *>;
+
 
 enum class FrameCategory : std::uint8_t {
     META,
@@ -140,69 +134,6 @@ enum class FrameCategory : std::uint8_t {
     STIM,
 };
 
-
-template<class T>
-struct generator {
-    struct promise_type {
-        T current_value_;
-        auto yield_value(T value) {
-            this->current_value_ = value;
-            return std::suspend_always{};
-        }
-        auto initial_suspend() { return std::suspend_always{}; }
-        auto final_suspend() noexcept { return std::suspend_always{}; }
-        generator get_return_object() { return generator{ this }; };
-        void unhandled_exception() { std::terminate(); }
-        void return_void() {}
-    };
-
-    using handle_t = std::coroutine_handle<promise_type>;
-
-    struct iterator {
-        handle_t coro_;
-        bool done_;
-
-        iterator() : done_(true) {}
-
-        explicit iterator(handle_t coro, bool done)
-            : coro_(coro), done_(done) {}
-
-        iterator& operator++() {
-            coro_.resume();
-            done_ = coro_.done();
-            return *this;
-        }
-
-        bool operator==(const iterator& rhs) const { return done_ == rhs.done_; }
-        bool operator!=(const iterator& rhs) const { return !(*this == rhs); }
-        const T& operator*() const { return coro_.promise().current_value_; }
-        const T* operator->() const { return &*(*this); }
-    };
-
-    iterator begin() {
-        coro_.resume();
-        return iterator(coro_, coro_.done());
-    }
-
-    iterator end() { return iterator(); }
-
-    generator(const generator&) = delete;
-    generator(generator&& rhs) : coro_(std::exchange(rhs.coro_, nullptr)) {}
-
-    ~generator() {
-        if (coro_) {
-            coro_.destroy();
-        }
-    }
-
-private:
-    explicit generator(promise_type* p)
-        : coro_(handle_t::from_promise(*p)) {}
-
-    handle_t coro_;
-};
-
-
 namespace detail
 {
     const std::string FILE_EXTENSION(".xmraw");
@@ -220,7 +151,7 @@ inline auto file_header_size() -> std::size_t {
 
 using rounding_func_t = std::function<std::size_t(std::size_t)>;
 
-const rounding_func_t create_rounding_func(std::size_t multiple) {
+inline const rounding_func_t create_rounding_func(std::size_t multiple) {
     return [=](std::size_t value) -> std::size_t {
         return (value + (multiple - 1)) & ~(multiple -1 );
     };
@@ -229,7 +160,7 @@ const rounding_func_t create_rounding_func(std::size_t multiple) {
 const auto round_to_alignment = create_rounding_func(__ALIGNMENT_REQUIREMENT);
 
 
-inline void _fcopy(blob_t * dest, const blob_t * src, std::size_t n)
+inline void _fcopy(char * dest, char const * src, std::size_t n)
 {
     for (std::size_t i = 0; i < n; ++i) {
         dest[i] = src[i];
@@ -241,14 +172,14 @@ inline void _fcopy(blob_t * dest, const blob_t * src, std::size_t n)
         return payload.get();
     }
 
-    payload_t create_payload(std::size_t size, blob_t const * data) {
-        auto pl = std::make_unique<blob_t[]>(size);
-        //auto pl = std::make_shared<blob_t[]>(size);
-        _fcopy(pl.get(), data, size);
+    inline payload_t create_payload(std::size_t size, blob_t const * data) {
+        //auto pl = std::make_unique<char[]>(size);
+        auto pl = std::make_shared<blob_t[]>(size);
+        _fcopy(reinterpret_cast<char*>(pl.get()), reinterpret_cast<char const*>(data), size);
         return pl;
     }
 #else
-    payload_t create_payload(std::size_t size, blob_t const * data) {
+	inline payload_t create_payload(std::size_t size, blob_t const * data) {
         return py::array_t<blob_t>(size, data);
     }
 
@@ -340,6 +271,58 @@ private:
 };
 
 
+/*
+ *
+ * Super simplicistic block memory manager.
+ *
+ */
+template <typename T, int _IS, int _NB>
+class BlockMemory {
+
+public:
+
+    using mem_block_t = T[_IS];
+
+    explicit BlockMemory() : m_memory{nullptr}, m_allocation_count{0} {
+        m_memory = new T[_IS * _NB];	// Ts to allocate: itemsize * number of items.
+        //printf("mem-base  : %p\n", m_memory);
+    }
+
+    ~BlockMemory() {
+        if (m_memory) {
+            delete[] m_memory;
+        }
+    }
+
+    T * acquire() noexcept {
+        const std::lock_guard<std::mutex> lock(m_mtx);
+
+        if (m_allocation_count >= _NB) {
+            return nullptr;
+        }
+        T * ptr = reinterpret_cast<T *>((m_memory + (m_allocation_count * _IS)));
+        //printf("acquire() %p\n", ptr);
+        m_allocation_count++;
+        return ptr;
+    }
+
+    void release() noexcept {
+        const std::lock_guard<std::mutex> lock(m_mtx);
+        //printf("release() %u\n", m_allocation_count);
+        if (m_allocation_count == 0) {
+            return;
+        }
+        m_allocation_count--;
+    }
+
+private:
+
+    T * m_memory;
+    std::size_t m_allocation_count;
+    std::mutex m_mtx;
+};
+
+
 /**
  */
 class XcpLogFileWriter
@@ -353,22 +336,22 @@ public:
             m_file_name.c_str(),
             GENERIC_READ | GENERIC_WRITE,
             0,
-            (LPSECURITY_ATTRIBUTES)NULL,
+            (LPSECURITY_ATTRIBUTES)nullptr,
             CREATE_ALWAYS,
             FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS,
-            NULL
+            nullptr
         );
         //printf("CreateFile returned: %u\n", GetLastError());
 #else
         m_fd = open(m_file_name.c_str(), O_CREAT | O_RDWR | O_TRUNC, 0666);
-
-        printf("open returned: %u\n", errno);
 #endif
         truncate(megabytes(prealloc));
         m_mmap = new mio::mmap_sink(m_fd);
         m_chunk_size = megabytes(chunk_size);
         m_intermediate_storage = new blob_t[m_chunk_size + megabytes(1)];
         m_offset = detail::FILE_HEADER_SIZE + detail::MAGIC_SIZE;
+
+        start_thread();
     }
 
     ~XcpLogFileWriter() {
@@ -378,10 +361,12 @@ public:
     void finalize() {
         if (!m_finalized) {
             m_finalized = true;
+            stop_thread();
             if (m_container_record_count) {
                 compress_frames();
             }
             write_header(detail::VERSION, 0x0000, m_num_containers, m_record_count, m_total_size_compressed, m_total_size_uncompressed);
+            m_mmap->unmap();
             truncate(m_offset);
 #if defined(_WIN32)
             CloseHandle(m_fd);
@@ -393,24 +378,28 @@ public:
         }
     }
 
-    void add_frame(uint8_t category, uint16_t counter, double timestamp, uint16_t length, const  payload_t& payload) {
-        frame_header_t frame {category, counter, timestamp, length};
+    void add_frame(uint8_t category, uint16_t counter, double timestamp, uint16_t length, char const * data) {
+        auto payload= new char[length];
+        //auto payload = mem.acquire();
 
-        store_im(&frame, sizeof(frame));
-        store_im(get_payload_ptr(payload), length);
-        m_container_record_count += 1;
-        m_container_size_uncompressed += (sizeof(frame) + length);
-        if (m_container_size_uncompressed > m_chunk_size) {
-            compress_frames();
-        }
+    	_fcopy(payload, data, length);
+    	my_queue.put(
+            std::make_tuple(category, counter, timestamp, length, payload)
+        );
     }
 
 protected:
     void truncate(off_t size) const
     {
+        //printf("truncating to: %lldKBytes.\n", kilobytes(size));
 #if defined(_WIN32)
-        SetFilePointer(m_fd, size, NULL, FILE_BEGIN);
-        SetEndOfFile(m_fd);
+
+        DWORD result;
+
+        result = SetFilePointer(m_fd, size, nullptr, FILE_BEGIN);
+
+        result = SetEndOfFile(m_fd);
+
 #else
         ftruncate(m_fd, size);
 #endif
@@ -418,11 +407,11 @@ protected:
 
     blob_t * ptr(std::size_t pos = 0) const
     {
-        return m_mmap->data() + pos;
+        return (blob_t *)(m_mmap->data() + pos);
     }
 
     void store_im(void const * data, std::size_t length) {
-        _fcopy(m_intermediate_storage + m_intermediate_storage_offset, (const blob_t*)data, length);
+        _fcopy(reinterpret_cast<char *>(m_intermediate_storage + m_intermediate_storage_offset), reinterpret_cast<char const*>(data), length);
         m_intermediate_storage_offset += length;
     }
 
@@ -430,7 +419,7 @@ protected:
         auto container = ContainerHeaderType{};
         //printf("Compressing %u frames... [%d]\n", m_container_record_count, m_intermediate_storage_offset);
         const int cp_size = ::LZ4_compress_default(
-            reinterpret_cast<blob_t*>(m_intermediate_storage), ptr(m_offset + detail::CONTAINER_SIZE),
+            reinterpret_cast<char const*>(m_intermediate_storage), reinterpret_cast<char *>(ptr(m_offset + detail::CONTAINER_SIZE)),
             m_intermediate_storage_offset, LZ4_COMPRESSBOUND(m_intermediate_storage_offset)
         );
         if (cp_size < 0) {
@@ -440,7 +429,7 @@ protected:
         container.record_count = m_container_record_count;
         container.size_compressed = cp_size;
         container.size_uncompressed = m_container_size_uncompressed;
-        _fcopy(ptr(m_offset), (blob_t*)&container, detail::CONTAINER_SIZE);
+        _fcopy(reinterpret_cast<char *>(ptr(m_offset)), reinterpret_cast<char const*>(&container), detail::CONTAINER_SIZE);
         m_offset += (detail::CONTAINER_SIZE + cp_size);
         m_total_size_uncompressed += m_container_size_uncompressed;
         m_total_size_compressed += cp_size;
@@ -452,9 +441,9 @@ protected:
         m_num_containers += 1;
     }
 
-    void write_bytes(std::size_t pos, std::size_t count, blob_t const * buf)
+    void write_bytes(std::size_t pos, std::size_t count, char const * buf) const
     {
-        auto addr = ptr(pos);
+        auto addr = reinterpret_cast<char *>(ptr(pos));
 
         _fcopy(addr, buf, count);
     }
@@ -470,25 +459,70 @@ protected:
         header.record_count = record_count;
         header.size_compressed = size_compressed;
         header.size_uncompressed = size_uncompressed;
-        write_bytes(0x00000000UL + detail::MAGIC_SIZE, detail::FILE_HEADER_SIZE, reinterpret_cast<blob_t *>(&header));
+        write_bytes(0x00000000UL + detail::MAGIC_SIZE, detail::FILE_HEADER_SIZE, reinterpret_cast<char const*>(&header));
+    }
+
+    bool start_thread() {
+        if (collector_thread.joinable()) {
+            return false;
+        }
+        stop_collector_thread_flag = false;
+        collector_thread = std::thread([this]() {
+            while (!stop_collector_thread_flag) {
+                auto item = my_queue.get();
+                const auto content = item.get();
+                if (stop_collector_thread_flag == true)
+                {
+                    break;
+                }
+                auto [category, counter, timestamp, length, payload] = content->value();
+                const frame_header_t frame{ category, counter, timestamp, length };
+
+                store_im(&frame, sizeof(frame));
+                store_im(payload, length);
+                delete[] payload;
+                //mem.release();
+                m_container_record_count += 1;
+                m_container_size_uncompressed += (sizeof(frame) + length);
+                if (m_container_size_uncompressed > m_chunk_size) {
+                    compress_frames();
+                }
+            }
+        });
+        return true;
+    }
+
+    bool stop_thread() {
+        if (!collector_thread.joinable()) {
+            return false;
+        }
+        stop_collector_thread_flag = true;
+        my_queue.put(std::nullopt);
+        collector_thread.join();
+        return true;
     }
 
 private:
     std::string m_file_name;
     std::size_t m_offset{0};
-    std::size_t m_chunk_size{0};
-    std::size_t m_num_containers{0};
-    std::size_t m_record_count{0};
-    std::size_t m_container_record_count{0};
-    std::size_t m_total_size_uncompressed{0};
-    std::size_t m_total_size_compressed{0};
-    std::size_t m_container_size_uncompressed{0};
-    std::size_t m_container_size_compressed{0};
+    std::uint32_t m_chunk_size{0};
+    std::uint32_t m_num_containers{0};
+    std::uint32_t m_record_count{0UL};
+    std::uint32_t m_container_record_count{0UL};
+    std::uint32_t m_total_size_uncompressed{0UL};
+    std::uint32_t m_total_size_compressed{0UL};
+    std::uint32_t m_container_size_uncompressed{0UL};
+    std::uint32_t m_container_size_compressed{0UL};
     __ALIGN blob_t * m_intermediate_storage{nullptr};
-    std::size_t m_intermediate_storage_offset{0};
+    std::uint32_t m_intermediate_storage_offset{0};
     mio::file_handle_type m_fd{INVALID_HANDLE_VALUE};
     mio::mmap_sink * m_mmap{nullptr};
     bool m_finalized{false};
+    std::thread collector_thread{};
+    std::mutex mtx;
+    TsQueue<std::optional<FrameTupleWriter>> my_queue;
+    BlockMemory<char, XCP_PAYLOAD_MAX, 16> mem{};
+    std::atomic_bool stop_collector_thread_flag{false};
 };
 
 
@@ -504,8 +538,7 @@ public:
         blob_t magic[detail::MAGIC_SIZE + 1];
 
         read_bytes(0ul, detail::MAGIC_SIZE, magic);
-        if (memcmp(detail::MAGIC.c_str(), magic, detail::MAGIC_SIZE))
-        {
+        if (memcmp(detail::MAGIC.c_str(), magic, detail::MAGIC_SIZE) != 0) {
             throw std::runtime_error("Invalid file magic.");
         }
         m_offset = detail::MAGIC_SIZE;
@@ -530,50 +563,62 @@ public:
         m_offset += detail::FILE_HEADER_SIZE;
     }
 
-    const FileHeaderType get_header()  const noexcept  {
+    [[nodiscard]]
+    FileHeaderType get_header()  const noexcept  {
         return m_header;
     }
 
-    generator<FrameTuple> next_record() /*noexcept */ {
+    [[nodiscard]]
+    auto get_header_as_tuple() const -> HeaderTuple {
+        auto hdr = get_header();
+
+        return std::make_tuple(
+            hdr.num_containers,
+            hdr.record_count,
+            hdr.size_uncompressed,
+            hdr.size_compressed,
+            ((std::uint64_t)(((double)hdr.size_uncompressed / (double)hdr.size_compressed * (double)100.0) + (double)0.5)) / 100.0
+        );
+    }
+
+    void reset() {
+        m_current_container = 0;
+        m_offset = file_header_size();
+    }
+
+
+    std::optional<FrameVector> next_block() /*noexcept*/ {
         auto container = ContainerHeaderType{};
         auto total = 0;
         auto frame = frame_header_t{};
         size_t boffs = 0;
         auto result = FrameVector{};
         payload_t payload;
-        uint32_t current_container{ 0 };
 
-        while (true) {
-            if (current_container >= m_header.num_containers) {
-                co_return;
-            }
-            read_bytes(m_offset, detail::CONTAINER_SIZE, reinterpret_cast<blob_t*>(&container));
-            //printf("rc: %d c: %d u: %d \n", container.record_count, container.size_compressed, container.size_uncompressed);
-
-            __ALIGN auto buffer = new blob_t[container.size_uncompressed];
-
-            m_offset += detail::CONTAINER_SIZE;
-            total += container.record_count;
-            result.reserve(container.record_count);
-            const int uc_size = ::LZ4_decompress_safe(ptr(m_offset), buffer, container.size_compressed, container.size_uncompressed);
-            if (uc_size < 0) {
-                throw std::runtime_error("LZ4 decompression failed.");
-            }
-            boffs = 0;
-            for (std::uint32_t idx = 0; idx < container.record_count; ++idx) {
-                _fcopy((blob_t*)&frame, &(buffer[boffs]), sizeof(frame_header_t));
-                boffs += sizeof(frame_header_t);
-                //result.emplace_back(std::make_tuple(frame.category, frame.counter, frame.timestamp, frame.length, create_payload(frame.length, &buffer[boffs])));
-                co_yield std::make_tuple(frame.category, frame.counter, frame.timestamp, frame.length, create_payload(frame.length, &buffer[boffs]));
-                boffs += frame.length;
-                //result.emplace_back(std::make_tuple(frame.category, frame.counter, frame.timestamp, frame.length, std::move(payload)));
-            }
-            m_offset += container.size_compressed;
-            current_container += 1;
-            delete[] buffer;
+        if (m_current_container >= m_header.num_containers) {
+            return std::nullopt;
         }
-        //printf("OK, retuning from next -- Total: %u\n", total);
-        //return std::optional<FrameVector>{result};
+        read_bytes(m_offset, detail::CONTAINER_SIZE, reinterpret_cast<blob_t*>(&container));
+        __ALIGN auto buffer = new blob_t[container.size_uncompressed];
+        m_offset += detail::CONTAINER_SIZE;
+        total += container.record_count;
+        result.reserve(container.record_count);
+        const int uc_size = ::LZ4_decompress_safe(reinterpret_cast<char const*>(ptr(m_offset)), reinterpret_cast<char *>(buffer), container.size_compressed, container.size_uncompressed);
+        if (uc_size < 0) {
+            throw std::runtime_error("LZ4 decompression failed.");
+        }
+        boffs = 0;
+        for (std::uint32_t idx = 0; idx < container.record_count; ++idx) {
+            _fcopy(reinterpret_cast<char *>(&frame), reinterpret_cast<char const*>(&(buffer[boffs])), sizeof(frame_header_t));
+            boffs += sizeof(frame_header_t);
+            result.emplace_back(std::make_tuple(frame.category, frame.counter, frame.timestamp, frame.length, create_payload(frame.length, &buffer[boffs])));
+            boffs += frame.length;
+        }
+        m_offset += container.size_compressed;
+        m_current_container += 1;
+        delete[] buffer;
+
+        return std::optional<FrameVector>{result};
     }
 
     ~XcpLogFileReader()
@@ -582,53 +627,24 @@ public:
     }
 
 protected:
+    [[nodiscard]]
     blob_t const *ptr(std::size_t pos = 0) const
     {
-        return m_mmap->data() + pos;
+        return reinterpret_cast<blob_t const*>(m_mmap->data() + pos);
     }
 
     void read_bytes(std::size_t pos, std::size_t count, blob_t * buf) const
     {
-        auto addr = ptr(pos);
-        _fcopy(buf, addr, count);
-    }
-
-    bool start_thread() {
-        if (decomp_thrd.joinable()) {
-            return false;
-        }
-        stop_thread_flag = false;
-        decomp_thrd = std::thread([this]() {
-            while (!stop_thread_flag) {
-            //    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-            //    if (cb_) cb_(1234);
-            }
-        });
-        return true;
-    }
-
-    bool stop_thread() {
-        if (!decomp_thrd.joinable()) {
-            return false;
-        }
-        stop_thread_flag = true;
-        decomp_thrd.join();
-        return true;
+        auto addr = reinterpret_cast<char const*>(ptr(pos));
+        _fcopy(reinterpret_cast<char *>(buf), addr, count);
     }
 
 private:
     std::string m_file_name;
     std::size_t m_offset{0};
+    std::size_t m_current_container{0};
     mio::mmap_source * m_mmap{nullptr};
-    //FileHeaderType m_header{0, 0, 0, 0, 0, 0, 0};
     FileHeaderType m_header;
-    std::thread decomp_thrd{};
-    std::mutex mtx;
-    std::queue<FrameVector> data_queue;
-    std::condition_variable data_cond;
-    std::atomic_bool stop_thread_flag{false};
-    std::atomic_bool request_decompression{false};
-    std::atomic_bool final_block{false};
 };
 
 #endif // __REKORDER_HPP
