@@ -1,5 +1,3 @@
-import pytest
-
 from pyxcp.transport.can import calculateFilter
 from pyxcp.transport.can import CAN_EXTENDED_ID
 from pyxcp.transport.can import Identifier
@@ -10,6 +8,8 @@ from pyxcp.transport.can import MAX_29_BIT_IDENTIFIER
 from pyxcp.transport.can import setDLC
 from pyxcp.transport.can import stripIdentifier
 
+import pytest
+
 
 def testSet0():
     assert setDLC(0) == 0
@@ -105,29 +105,26 @@ def testfilter3():
 
 
 def testfilter4():
-    assert (
-        calculateFilter(
-            [
-                0x1560 | CAN_EXTENDED_ID,
-                0x1561,
-                0x1562,
-                0x1563,
-                0x1564,
-                0x1565,
-                0x1566,
-                0x1567,
-                0x1568,
-                0x1569,
-                0x156A,
-                0x156B,
-                0x156C,
-                0x156D,
-                0x1563,
-                0x156F,
-            ]
-        )
-        == (0x1560, 0x1FFFFFF0)
-    )
+    assert calculateFilter(
+        [
+            0x1560 | CAN_EXTENDED_ID,
+            0x1561,
+            0x1562,
+            0x1563,
+            0x1564,
+            0x1565,
+            0x1566,
+            0x1567,
+            0x1568,
+            0x1569,
+            0x156A,
+            0x156B,
+            0x156C,
+            0x156D,
+            0x1563,
+            0x156F,
+        ]
+    ) == (0x1560, 0x1FFFFFF0)
 
 
 def testfilter5():
